/* START LIBRARY DESCRIPTION ****************************************************
FILE: 			Display NHD-0220D3Z.c
DESCRIPTION:	Functions for Newhaven NHD-0220D3Z-xxx-xxx 2-line 20 char display
END DESCRIPTION ****************************************************************/


/********* Function PROTOTYPES (follow syntax, including comment lines) ***************************/
/*** BeginHeader DisplayBoot */
void DisplayBoot(void);
void DisplayWrite (char *Line1, char *Line2, int Brightness);
void DisplayContrast (char* Contrast);

#ifndef BRIGHT
	#define BRIGHT 	8									 //when connected to host, display brightness (1-8).
#endif
#ifndef DIM
	#define DIM    	2								//when not connected to host, display brightness (1-8).
#endif
#ifndef CONTRAST
	#define CONTRAST 	42						  							 //Set default contrast (1-50, 50=high).
#endif
/*** EndHeader */

#ifndef DO_DEBUG
	#nodebug							   // disables insertion of breakpoints for debugging, this file only.
#endif


/*********************************** Function DisplayBoot *****************************************/
void DisplayBoot (void)  {	//boot the Newhaven Displays NHD-0220D3Z-xxx-xxx (2-line, 20 char)
	serDopen (9600);																	//CONFIGURE RABBIT SERIAL PORT.
	serDdatabits (PARAM_8BIT);
	serDparity (PARAM_NOPARITY);

	serDputc(0xFE);  serDputc(0x52);  serDputc(CONTRAST);	 									 //SET CONTRAST.
}


/*********************************** Write to the Display *****************************************/
void DisplayWrite (char *Line1, char *Line2, int Brightness)  {
/* START FUNCTION DESCRIPTION *********************************************
DisplayWrite       <Display NHD-0220D3Z.c>
DESCRIPTION: With the Newhaven Displays NHD-0220D3Z-xxx-xxx, display 2
	lines of 20 characters.  Note that there must be a pause of at least a
	few mS before setting brightness.
ENTER:  Pointers to line 1 and line 2 null terminated strings.
        Also, backlighting from 1 to 8.  No error checking is done to
        string length or brightness level!
END DESCRIPTION **********************************************************/
	serDputc(0xFE);  serDputc(0x53);  serDputc(Brightness);  Pause (10);			  //ADJUST BRIGHTNESS.

	serDputc(0xFE);  serDputc(0x51);  Pause (10);												//clear display.
	serDwrite(Line1, strlen(Line1));															 //display first line.
	serDputc(0xFE);  serDputc(0x45);  serDputc(0x40);						  //go to start of second line.
	serDwrite(Line2, strlen(Line2));															//display second line.
	Pause(10);
}


/*********************************** Adjust Display Contrast **************************************/
void DisplayContrast (char* Contrast)  {
/* START FUNCTION DESCRIPTION ********************
DisplayContrast       <Display NHD-0220D3Z.c>
DESCRIPTION:	Adjust the display contrast
PARAMETER 1:	Contrast, 0 to 99.  This display
					can only use 1 to 50
END DESCRIPTION *********************************/
	unsigned int Con;

	Con = (unsigned int) Contrast[0];
	Con /= 2;										 //adjust user's 0 to 99 range to display's 1 to 50 range.
	if (Con > 50)  Con = 50;
	if (!Con)      Con = 1;
	serCputc(0xFE);  serCputc(0x52);  serCputc(Con);		 									 //SET CONTRAST.
	Pause(10);
}
/**************************************************************************************************/

