/* START LIBRARY DESCRIPTION ************************************
FILE:				DiffCon Command.c
PARENT FILE:	DiffCon.c
DESCRIPTION:	Housekeeping and command parsing functions.
END DESCRIPTION ************************************************/


/********* Function PROTOTYPES (follow syntax, including comment lines) ***************************/
/*** BeginHeader CommandHandler, etc */
void CommandHandler (char* Cmd, int InLen);
void InvalidCommand (char* Cmd);
/*** EndHeader ***********************/

#ifndef DO_DEBUG
	#nodebug								// disables insertion of breakpoints for debugging, this file only.
#endif


/*************************** Incoming Command From Host *******************************************/
void CommandHandler(char* Cmd, int InLen)  {
/* START FUNCTION DESCRIPTION ******************************************
CommandHandler    <ChromaMatic2 Commands.c>
DESCRIPTION:	Some command data has arrived from host.  Process it.
PARAMETER 1:	char* Cmd, incoming data, will be processed.
PARAMETER 2:   int Len, the length of the command and packet.
COMMENTS:		Assumed to have one command per packet. (New to Rabbit Command Handler)
END DESCRIPTION *******************************************************/
	char PllLock[1], C[10];												//a channel data to be sent to the FPGA.
	int  PhiDel, Dat;
	float Period, Val;

	switch (Cmd[0])  {
		case 'H':																  //ECHO INCOMING HEARTBEAT (1 byte).
			if (InLen == 1)																				//valid command.
				udp_send(&HostSock, Cmd, 1);													 //echo the heartbeat.
			else  InvalidCommand (Cmd);
			break;

		case 'F':																//SET OSCILLATOR FREQUENCY (5 bytes).
			if (InLen == 5)  {																			//valid command.
				strncpy (Buf1, Cmd+1, 4);													 //extract the frequency.
				Buf1[4] = 0x0;
				Freq = atoi(Buf1);
				if (Freq < FREQ_MIN)  Freq = FREQ_MIN;			 					  						//minimum.
				if (Freq > FREQ_MAX)  Freq = FREQ_MAX;	 						  							//maximum.
				FreqSet (Freq);
			}
			else  InvalidCommand (Cmd);
			break;

		case 'A':																			 //SET AC VOLTAGE (3 bytes).
			if (InLen == 3)																				//valid command.
				PotSet (Cmd[1], Cmd[2]);																 //set the pots.
			else  InvalidCommand (Cmd);
			break;

		case 'G':															//SET AC VOLTAGE GAIN DECADES (3 bytes).
			if (InLen == 3)  {																			//valid command.
				if (Cmd[1] == '3')  RelayLatch(2);									 //Set AC-V Gain Relay To 3.
				else                RelayLatch(1);									 //set AC-V gain relay to 1.
				switch (Cmd[2])  {																	 //Set Gain Decade.
					case '0':																					  //Gain = 1.
						BitWrPortI (PADR, &PADRShadow, 0, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 0, 7);		 					 	  //set A1 (SPI data).
						break;
					case '1':																					 //Gain = 10.
						BitWrPortI (PADR, &PADRShadow, 1, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 0, 7);		 					 	  //set A1 (SPI data).
						break;
					case '2':																					//Gain = 100.
						BitWrPortI (PADR, &PADRShadow, 0, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 1, 7);		 					 	  //set A1 (SPI data).
						break;
					case '3':																				 //Gain = 1,000.
						BitWrPortI (PADR, &PADRShadow, 1, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 1, 7);		 					 	  //set A1 (SPI data).
						break;
					default:
						InvalidCommand (Cmd);
						break;
				}
				BitWrPortI (PADR, &PADRShadow, 0, 1);							 					 //set !AC-V_CS.
				BitWrPortI (PADR, &PADRShadow, 1, 1);							 				  //clear !AC-V_CS.
			}
			else  InvalidCommand (Cmd);
			break;

		case 'C':															//SET AC CURRENT GAIN DECADES (3 bytes).
			if (InLen == 3)  {																			//valid command.
				if (Cmd[1] == '3')  RelayLatch(8);									 //Set AC-I Gain Relay To 3.
				else                RelayLatch(4);									 //set AC-I gain relay to 1.
				switch (Cmd[2])  {																	 //Set Gain Decade.
					case '0':																					  //Gain = 1.
						BitWrPortI (PADR, &PADRShadow, 0, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 0, 7);		 					 	  //set A1 (SPI data).
						break;
					case '1':																					 //Gain = 10.
						BitWrPortI (PADR, &PADRShadow, 1, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 0, 7);		 					 	  //set A1 (SPI data).
						break;
					case '2':																					//Gain = 100.
						BitWrPortI (PADR, &PADRShadow, 0, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 1, 7);		 					 	  //set A1 (SPI data).
						break;
					case '3':																				 //Gain = 1,000.
						BitWrPortI (PADR, &PADRShadow, 1, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 1, 7);		 					 	  //set A1 (SPI data).
						break;
					default:
						InvalidCommand (Cmd);
						break;
				}
				BitWrPortI (PADR, &PADRShadow, 0, 0);							 					 //set !AC-I_CS.
				BitWrPortI (PADR, &PADRShadow, 1, 0);							 				  //clear !AC-I_CS.
			}
			else  InvalidCommand (Cmd);
			break;

		case 'D':																			 //SET DC VOLTAGE (7 bytes).
			if (InLen == 7)  {																			//valid command.
				strncpy (Buf1, Cmd+1, 6);													 //extract the frequency.
				Buf1[6] = 0x0;
				Val = atof(Buf1);													  //convert the string to a float.
				DacSet (Val);								  //error handling (outside of 1) is done here too.
			}
			else  InvalidCommand (Cmd);
			break;

		case 'S':																	 //READ SATURATION FLAGS (1 byte).
			if (InLen == 1)  {																			//valid command.
				sprintf (Buf1, "S%c", Saturation);
				udp_send(&HostSock, Buf1, 2);													 //echo the heartbeat.
				Saturation = 0;
			}
			else  InvalidCommand (Cmd);
			break;

		case 'Q':																//SET SAMPLES IN AVERAGING (5 bytes).
			if (InLen == 5)  {																			//valid command.
				strncpy (Buf1, Cmd+1, 4);													 //extract the frequency.
				Buf1[4] = 0x0;
				Dat = atoi(Buf1);													  //convert the string to integer.
				if (Dat > 0 && Dat < 10000)  SamplesToAvg = Dat;
			}
			else  InvalidCommand (Cmd);
			break;

		case '2':														  //READ ALL VOLTAGES AND CURRENTS AT 270.
			if (InLen == 1)  {																			//valid command.
				Period = TIMER_FREQ / (float)Freq;
				PhiDel  = (int)(Period * PHI270);										 //Calculate Phase Delay.
			   WrPortI (TCS3HR, NULL, PhiDel / 256);	 //ADC Convert high: Timer C3 set MS (138.889uS).
	   		WrPortI (TCS3LR, NULL, PhiDel % 256);							 //Timer C3 set LS (542.535nS).
	   		PhiDel += TIMER_RST_DELAY;									 //keep going, to prevent retriggers.
	   		WrPortI (TCR3HR, NULL, PhiDel / 256);	//ADC Convert low: Timer C3 reset MS (138.889uS).
	   		WrPortI (TCR3LR, NULL, PhiDel % 256);		              //Timer C3 reset LS (542.535nS).
	   		PhiDel ++;																 //loop back Timer C bit later.
	         PhaseMS = PhiDel / 256;											//Timer C limit when phase timing.
	         PhaseLS = PhiDel % 256;
	         AvgCount = SamplesToAvg;											  //initiate averaging counter.
	         DoAdcs   = 1;																				 //request ADCs.
	      }
			else  InvalidCommand (Cmd);
			break;

		case '9':															//READ ALL VOLTAGES AND CURRENTS AT 90.
			if (InLen == 1)  {																			//valid command.
				Period = TIMER_FREQ / (float)Freq;
				PhiDel  = (int)(Period * PHI90);											 //Calculate Phase Delay.
			   WrPortI (TCS3HR, NULL, PhiDel / 256);	 //ADC Convert high: Timer C3 set MS (138.889uS).
	   		WrPortI (TCS3LR, NULL, PhiDel % 256);							 //Timer C3 set LS (542.535nS).
	   		PhiDel += TIMER_RST_DELAY;									 //keep going, to prevent retriggers.
	   		WrPortI (TCR3HR, NULL, PhiDel / 256);	//ADC Convert low: Timer C3 reset MS (138.889uS).
	   		WrPortI (TCR3LR, NULL, PhiDel % 256);		              //Timer C3 reset LS (542.535nS).
	   		PhiDel ++;																 //loop back Timer C bit later.
	         PhaseMS = PhiDel / 256;											//Timer C limit when phase timing.
	         PhaseLS = PhiDel % 256;
	         AvgCount = SamplesToAvg;											  //initiate averaging counter.
	         DoAdcs   = 1;																				 //request ADCs.
	      }
			else  InvalidCommand (Cmd);
			break;

		default:  {
			sprintf(Buf1, "ERROR: %s received invalid command (0x%X).\n", UNIT_NAME, Cmd[0]);
			udp_send(&MonitorSock, Buf1, strlen(Buf1));
			break;
		}
	}
}


/*************************** ERROR: Invalid Command From Host *************************************/
void InvalidCommand (char* Cmd)  {  //in CommandHandler(), when a command does not have enough data.
	sprintf(Buf1, "ERROR: %s received an invalid command (0x%X).\n", UNIT_NAME, Cmd[0]);
	udp_send(&MonitorSock, Buf1, strlen(Buf1));
}

/**************************************************************************************************/